@@ -1,6 +1,7 @@
 /*
     tc_client, a simple non-flash client for tinychat(.com)
     Copyright (C) 2014  alicia@ion.nu
+    Copyright (C) 2014-2015  Jade Lea
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Affero General Public License as published by
@@ -28,5 +29,5 @@ extern void idlist_add(int id, const char* name);
 extern void idlist_remove(const char* name);
 extern void idlist_rename(const char* oldname, const char* newname);
 extern int idlist_get(const char* name);
-extern void idlist_set_op(const char* name);
+extern void idlist_set_op(const char* name, char op);
 extern char idlist_is_op(const char* name);
